@@ -62,8 +62,8 @@ use std::collections::{HashMap, HashSet};
 use std::convert::{TryFrom, TryInto};
 use std::result::Result as StdResult;
 
-/// Represents both sequences received as input of the PostProcessor
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+/// Represents any sequences received as input of the PostProcessor
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize, Eq)]
 pub enum Sequence {
     /// This is the first sequence, the one that is always specified
     A,
@@ -91,7 +91,7 @@ pub enum Sequence {
 ///
 /// [`SpecialToken`]: struct.SpecialToken.html
 ///
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize, Eq)]
 pub enum Piece {
     Sequence { id: Sequence, type_id: u32 },
     SpecialToken { id: String, type_id: u32 },
@@ -188,7 +188,7 @@ impl TryFrom<&str> for Piece {
 ///     vec!["A".into(), "complex".into(), "special".into(), "token".into(), ":".into()]
 /// ).unwrap();
 /// ```
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize, Eq)]
 pub struct SpecialToken {
     /// A unique id used to identify this SpecialToken in the template
     id: String,
@@ -249,7 +249,7 @@ impl SpecialToken {
 ///
 /// [`Piece`]: enum.Piece.html
 ///
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize, Eq)]
 #[serde(transparent)]
 pub struct Template(Vec<Piece>);
 
@@ -289,7 +289,7 @@ impl TryFrom<&str> for Template {
 /// from a HashMap or a Vec<[`SpecialToken`]>.
 ///
 /// [`SpecialToken`]: struct.SpecialToken.html
-#[derive(Debug, Clone, PartialEq, Default, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Default, Serialize, Deserialize, Eq)]
 #[serde(transparent)]
 pub struct Tokens(
     #[serde(serialize_with = "crate::utils::ordered_map")] pub HashMap<String, SpecialToken>,
@@ -332,7 +332,7 @@ impl From<HashMap<String, SpecialToken>> for Tokens {
 ///     .unwrap();
 /// ```
 ///
-#[derive(Debug, Clone, PartialEq, Builder, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Builder, Serialize, Deserialize, Eq)]
 #[serde(tag = "type", from = "TemplateProcessingDeserializer")]
 #[builder(build_fn(validate = "Self::validate"))]
 pub struct TemplateProcessing {
@@ -479,145 +479,102 @@ impl TemplateProcessing {
     fn apply_template(
         &self,
         template: &[Piece],
-        mut encoding: Encoding,
-        mut pair: Option<Encoding>,
+        mut encodings: Vec<Encoding>,
         add_special_tokens: bool,
-    ) -> Result<Encoding> {
-        // Compute the new size
-        let mut new_len = 0;
-        for piece in template {
-            new_len += match piece {
-                Piece::Sequence {
-                    id: Sequence::A, ..
-                } => encoding.len(),
-                Piece::Sequence {
-                    id: Sequence::B, ..
-                } => pair
-                    .as_ref()
-                    .ok_or("Template expected a pair sequence, but none provided")?
-                    .len(),
-                Piece::SpecialToken { id, .. } => {
-                    if add_special_tokens {
-                        self.special_tokens
-                            .0
-                            .get(id)
-                            .ok_or_else(|| format!("Missing SpecialToken with id {}", id))?
-                            .ids
-                            .len()
-                    } else {
-                        0
+    ) -> Result<Vec<Encoding>> {
+        let final_encodings: Vec<Encoding> = template
+            .iter()
+            .flat_map(|piece| {
+                match piece {
+                    Piece::Sequence { id, type_id } => {
+                        let i = if *id == Sequence::A { 0 } else { 1 };
+                        let encoding = &mut encodings[i];
+                        encoding.set_type_ids(vec![*type_id; encoding.len()]);
+                        encoding.set_sequence_id(i);
+                        Some(encoding.clone())
+                    }
+                    Piece::SpecialToken { id, type_id } => {
+                        if add_special_tokens {
+                            let tok = &self.special_tokens.0[id]; // We already checked existance above
+                            let len = tok.ids.len();
+
+                            let encoding = Encoding::new(
+                                tok.ids.clone(),
+                                std::iter::repeat(*type_id).take(len).collect(),
+                                tok.tokens.clone(),
+                                // words
+                                std::iter::repeat(None).take(len).collect(),
+                                // offsets
+                                std::iter::repeat((0, 0)).take(len).collect(),
+                                // special_tokens_mask
+                                std::iter::repeat(1).take(len).collect(),
+                                // attention_mask
+                                std::iter::repeat(1).take(len).collect(),
+                                // overflowing
+                                vec![],
+                                // sequence_range
+                                HashMap::new(),
+                            );
+                            Some(encoding)
+                        } else {
+                            None
+                        }
                     }
                 }
-            };
-        }
-
-        // Then build the new Encoding
-        let mut ids = Vec::with_capacity(new_len);
-        let mut type_ids = Vec::with_capacity(new_len);
-        let mut tokens = Vec::with_capacity(new_len);
-        let mut words = Vec::with_capacity(new_len);
-        let mut offsets = Vec::with_capacity(new_len);
-        let mut special_tokens_mask = Vec::with_capacity(new_len);
-        let mut attention_mask = Vec::with_capacity(new_len);
-        let mut sequence_ranges = HashMap::new();
-
-        let pair_overflowing = pair.as_mut().map_or(vec![], |e| e.take_overflowing());
-        let mut overflowing = encoding
-            .take_overflowing()
-            .into_iter()
-            .flat_map(|encoding| {
-                // 1. The pair itself
-                let mut overflowings = vec![self.apply_template(
-                    template,
-                    encoding.clone(),
-                    pair.clone(),
-                    add_special_tokens,
-                )];
-
-                // 2. Its overflowings
-                for other_o in &pair_overflowing {
-                    overflowings.push(self.apply_template(
-                        template,
-                        encoding.clone(),
-                        Some(other_o.clone()),
-                        add_special_tokens,
-                    ));
-                }
-
-                overflowings
             })
-            .collect::<Result<Vec<_>>>()?;
-        // We also need to combine the first sequence with all other overflowings
-        overflowing.extend(
-            pair_overflowing
-                .into_iter()
-                .map(|pair| {
-                    self.apply_template(template, encoding.clone(), Some(pair), add_special_tokens)
-                })
-                .collect::<Result<Vec<_>>>()?,
-        );
+            .collect();
 
-        for piece in template {
-            match piece {
-                Piece::Sequence {
-                    id: Sequence::A,
-                    type_id,
-                } => {
-                    let seq_start = ids.len();
-                    let seq_end = seq_start + encoding.len();
-                    sequence_ranges.insert(0, seq_start..seq_end);
-                    ids.extend(encoding.get_ids());
-                    type_ids.extend(std::iter::repeat(type_id).take(encoding.len()));
-                    tokens.extend(encoding.get_tokens().iter().map(|s| s.to_owned()));
-                    words.extend(encoding.get_word_ids());
-                    offsets.extend(encoding.get_offsets());
-                    special_tokens_mask.extend(encoding.get_special_tokens_mask());
-                    attention_mask.extend(encoding.get_attention_mask());
-                }
-                Piece::Sequence {
-                    id: Sequence::B,
-                    type_id,
-                } => {
-                    let pair = pair.as_ref().expect("Missing pair sequence, checked above");
-                    let seq_start = ids.len();
-                    let seq_end = seq_start + pair.len();
-                    sequence_ranges.insert(1, seq_start..seq_end);
-                    ids.extend(pair.get_ids());
-                    type_ids.extend(std::iter::repeat(type_id).take(pair.len()));
-                    tokens.extend(pair.get_tokens().iter().map(|s| s.to_owned()));
-                    words.extend(pair.get_word_ids());
-                    offsets.extend(pair.get_offsets());
-                    special_tokens_mask.extend(pair.get_special_tokens_mask());
-                    attention_mask.extend(pair.get_attention_mask());
-                }
-                Piece::SpecialToken { id, type_id } => {
-                    if add_special_tokens {
-                        let tok = &self.special_tokens.0[id]; // We already checked existance above
-                        let len = tok.ids.len();
+        //let mut pair = if encodings.len() > 1 {
+        //    Some(encodings.pop().unwrap())
+        //} else {
+        //    None
+        //};
+        //let mut encoding = encodings.pop().unwrap();
 
-                        ids.extend(&tok.ids);
-                        type_ids.extend(std::iter::repeat(type_id).take(len));
-                        tokens.extend(tok.tokens.clone());
-                        words.extend(std::iter::repeat(None).take(len));
-                        offsets.extend(std::iter::repeat((0, 0)).take(len));
-                        special_tokens_mask.extend(std::iter::repeat(1).take(len));
-                        attention_mask.extend(std::iter::repeat(1).take(len));
-                    }
-                }
-            }
-        }
+        //let pair_overflowing = pair.as_mut().map_or(vec![], |e| e.take_overflowing());
+        //let mut overflowing: Vec<Encoding> = encoding
+        //    .take_overflowing()
+        //    .iter()
+        //    .map(|encoding| -> Result<Vec<Encoding>> {
+        //        // 1. The pair itself
+        //        let mut overflowings = self.apply_template(
+        //            template,
+        //            if encodings.len() > 1 {
+        //                vec![encoding.clone(), encodings[1].clone()]
+        //            } else {
+        //                vec![encoding.clone()]
+        //            },
+        //            add_special_tokens,
+        //        )?;
 
-        Ok(Encoding::new(
-            ids,
-            type_ids,
-            tokens,
-            words,
-            offsets,
-            special_tokens_mask,
-            attention_mask,
-            overflowing,
-            sequence_ranges,
-        ))
+        //        // 2. Its overflowings
+        //        for other_o in &pair_overflowing {
+        //            overflowings.extend(self.apply_template(
+        //                template,
+        //                vec![encoding.clone(), other_o.clone()],
+        //                add_special_tokens,
+        //            )?);
+        //        }
+
+        //        Ok(overflowings)
+        //    })
+        //    .collect::<Result<Vec<Vec<Encoding>>>>()?
+        //    .into_iter()
+        //    .flatten()
+        //    .collect();
+        //// We also need to combine the first sequence with all other overflowings
+        //overflowing.extend(
+        //    pair_overflowing
+        //        .into_iter()
+        //        .map(|pair| {
+        //            self.apply_template(template, vec![encoding.clone(), pair], add_special_tokens)
+        //        })
+        //        .collect::<Result<Vec<_>>>()?
+        //        .into_iter()
+        //        .flatten(),
+        //);
+
+        Ok(final_encodings)
     }
 }
 
@@ -630,22 +587,36 @@ impl PostProcessor for TemplateProcessing {
         }
     }
 
-    fn process(
+    fn process_encodings(
         &self,
-        encoding: Encoding,
-        pair: Option<Encoding>,
+        encodings: Vec<Encoding>,
         add_special_tokens: bool,
-    ) -> Result<Encoding> {
-        self.apply_template(
-            if pair.is_some() {
-                &self.pair.0
-            } else {
-                &self.single.0
-            },
-            encoding,
-            pair,
-            add_special_tokens,
-        )
+    ) -> Result<Vec<Encoding>> {
+        // let (encoding, pair): (Encoding, Option<Encoding>) = match encodings.len() {
+        //     1 => (
+        //         encodings
+        //             .pop()
+        //             .ok_or(ProcessorError::InvalidEncodingsVecLength)?,
+        //         None,
+        //     ),
+        //     2 => {
+        //         let pair = encodings
+        //             .pop()
+        //             .ok_or(ProcessorError::InvalidEncodingsVecLength)?;
+        //         let encoding = encodings
+        //             .pop()
+        //             .ok_or(ProcessorError::InvalidEncodingsVecLength)?;
+        //         (encoding, Some(pair))
+        //     }
+        //     _ => return Err(Box::new(ProcessorError::InvalidEncodingsVecLength)),
+        // };
+        let template = match encodings.len() {
+            2 => &self.pair.0,
+            1 => &self.single.0,
+            _ => todo!(),
+        };
+        let encodings = self.apply_template(template, encodings, add_special_tokens)?;
+        Ok(encodings)
     }
 }
 
@@ -865,7 +836,6 @@ mod tests {
         );
         let pair = Encoding::from_tokens(vec![Token::new(15, "pair".into(), (0, 4))], 0);
         let single_encoding = processor.process(encoding.clone(), None, true).unwrap();
-        let pair_encoding = processor.process(encoding, Some(pair), true).unwrap();
         assert_eq!(
             single_encoding,
             Encoding::new(
@@ -887,6 +857,7 @@ mod tests {
         );
         assert_eq!(single_encoding.token_to_sequence(2), Some(0));
         assert_eq!(single_encoding.token_to_sequence(3), None);
+        let pair_encoding = processor.process(encoding, Some(pair), true).unwrap();
         assert_eq!(
             pair_encoding,
             Encoding::new(
@@ -914,6 +885,182 @@ mod tests {
         assert_eq!(pair_encoding.token_to_sequence(5), None);
     }
 
+    #[test]
+    fn template_processing_overflowing() {
+        let processor = tests::get_bert_template();
+        assert_eq!(processor.added_tokens(false), 2);
+        assert_eq!(processor.added_tokens(true), 3);
+
+        use crate::Token;
+        let mut encoding = Encoding::from_tokens(
+            vec![
+                Token::new(12, "Hello".into(), (0, 5)),
+                Token::new(14, "there".into(), (6, 11)),
+            ],
+            0,
+        );
+        let overflowing = Encoding::from_tokens(vec![Token::new(13, "you".into(), (12, 15))], 0);
+        encoding.set_overflowing(vec![overflowing]);
+
+        let mut pair = Encoding::from_tokens(
+            vec![
+                Token::new(15, "pair".into(), (0, 4)),
+                Token::new(16, "with".into(), (5, 9)),
+            ],
+            0,
+        );
+        let pair_overflowing =
+            Encoding::from_tokens(vec![Token::new(17, "info".into(), (10, 14))], 0);
+        pair.set_overflowing(vec![pair_overflowing]);
+
+        let single_encoding = processor.process(encoding.clone(), None, true).unwrap();
+        assert_eq!(
+            single_encoding,
+            Encoding::new(
+                vec![1, 12, 14, 0],
+                vec![0, 0, 0, 0],
+                vec![
+                    "[CLS]".into(),
+                    "Hello".into(),
+                    "there".into(),
+                    "[SEP]".into()
+                ],
+                vec![None, None, None, None],
+                vec![(0, 0), (0, 5), (6, 11), (0, 0)],
+                vec![1, 0, 0, 1],
+                vec![1, 1, 1, 1],
+                vec![Encoding::new(
+                    vec![1, 13, 0],
+                    vec![0, 0, 0],
+                    vec!["[CLS]".into(), "you".into(), "[SEP]".into()],
+                    vec![None, None, None],
+                    vec![(0, 0), (12, 15), (0, 0)],
+                    vec![1, 0, 1],
+                    vec![1, 1, 1],
+                    vec![],
+                    HashMap::from_iter(vec![(0, 1..2)]),
+                )],
+                HashMap::from_iter(vec![(0, 1..3)]),
+            )
+        );
+        assert_eq!(single_encoding.token_to_sequence(2), Some(0));
+        assert_eq!(single_encoding.token_to_sequence(3), None);
+        let pair_encoding = processor.process(encoding, Some(pair), true).unwrap();
+        println!("{pair_encoding:#?}");
+        assert_eq!(
+            pair_encoding,
+            Encoding::new(
+                vec![1, 12, 14, 0, 15, 16, 0],
+                vec![0, 0, 0, 0, 1, 1, 1],
+                vec![
+                    "[CLS]".into(),
+                    "Hello".into(),
+                    "there".into(),
+                    "[SEP]".into(),
+                    "pair".into(),
+                    "with".into(),
+                    "[SEP]".into()
+                ],
+                vec![None, None, None, None, None, None, None],
+                vec![(0, 0), (0, 5), (6, 11), (0, 0), (0, 4), (5, 9), (0, 0)],
+                vec![1, 0, 0, 1, 0, 0, 1],
+                vec![1, 1, 1, 1, 1, 1, 1],
+                vec![
+                    Encoding::new(
+                        vec![1, 13, 0, 15, 16, 0],
+                        vec![0, 0, 0, 1, 1, 1],
+                        vec![
+                            "[CLS]".into(),
+                            "you".into(),
+                            "[SEP]".into(),
+                            "pair".into(),
+                            "with".into(),
+                            "[SEP]".into()
+                        ],
+                        vec![None, None, None, None, None, None],
+                        vec![(0, 0), (12, 15), (0, 0), (0, 4), (5, 9), (0, 0)],
+                        vec![1, 0, 1, 0, 0, 1],
+                        vec![1, 1, 1, 1, 1, 1],
+                        vec![Encoding::new(
+                            vec![1, 13, 0, 17, 0],
+                            vec![0, 0, 0, 0, 1],
+                            vec![
+                                "[CLS]".into(),
+                                "you".into(),
+                                "[SEP]".into(),
+                                "info".into(),
+                                "[SEP]".into()
+                            ],
+                            vec![None, None, None, None, None,],
+                            vec![(0, 0), (12, 15), (0, 0), (10, 14), (0, 0)],
+                            vec![1, 0, 1, 0, 1],
+                            vec![1, 1, 1, 1, 1],
+                            vec![],
+                            HashMap::from_iter(vec![(0, 1..2), (1, 3..4)]),
+                        ),],
+                        HashMap::from_iter(vec![(1, 3..5), (0, 1..2)]),
+                    ),
+                    Encoding::new(
+                        vec![1, 13, 0, 17, 0],
+                        vec![0, 0, 0, 0, 1],
+                        vec![
+                            "[CLS]".into(),
+                            "you".into(),
+                            "[SEP]".into(),
+                            "info".into(),
+                            "[SEP]".into()
+                        ],
+                        vec![None, None, None, None, None,],
+                        vec![(0, 0), (12, 15), (0, 0), (10, 14), (0, 0)],
+                        vec![1, 0, 1, 0, 1],
+                        vec![1, 1, 1, 1, 1],
+                        vec![],
+                        HashMap::from_iter(vec![(0, 1..2), (1, 3..4)]),
+                    ),
+                    Encoding::new(
+                        vec![1, 12, 14, 0, 17, 0],
+                        vec![0, 0, 0, 0, 0, 1],
+                        vec![
+                            "[CLS]".into(),
+                            "Hello".into(),
+                            "there".into(),
+                            "[SEP]".into(),
+                            "info".into(),
+                            "[SEP]".into()
+                        ],
+                        vec![None, None, None, None, None, None],
+                        vec![(0, 0), (0, 5), (6, 11), (0, 0), (10, 14), (0, 0)],
+                        vec![1, 0, 0, 1, 0, 1],
+                        vec![1, 1, 1, 1, 1, 1],
+                        vec![Encoding::new(
+                            vec![1, 13, 0, 17, 0],
+                            vec![0, 0, 0, 0, 1],
+                            vec![
+                                "[CLS]".into(),
+                                "you".into(),
+                                "[SEP]".into(),
+                                "info".into(),
+                                "[SEP]".into()
+                            ],
+                            vec![None, None, None, None, None,],
+                            vec![(0, 0), (12, 15), (0, 0), (10, 14), (0, 0)],
+                            vec![1, 0, 1, 0, 1],
+                            vec![1, 1, 1, 1, 1],
+                            vec![],
+                            HashMap::from_iter(vec![(0, 1..2), (1, 3..4)]),
+                        ),],
+                        HashMap::from_iter(vec![(0, 1..3), (1, 4..5)]),
+                    )
+                ],
+                HashMap::from_iter(vec![(0, 1..3), (1, 4..6)]),
+            )
+        );
+        assert_eq!(pair_encoding.token_to_sequence(2), Some(0));
+        assert_eq!(pair_encoding.token_to_sequence(3), None);
+        assert_eq!(pair_encoding.token_to_sequence(4), Some(1));
+        assert_eq!(pair_encoding.token_to_sequence(5), Some(1));
+        assert_eq!(pair_encoding.token_to_sequence(6), None);
+    }
     #[test]
     fn pair_must_use_both_sequences() {
         let processor = TemplateProcessing::builder()
